/*
 * ForceFieldJoint.cpp
 *
 * Copyright Notice: see copyright.txt
 *
 *  Date: 3/8/2012 (D/M/Y)
 *  Author: Dmitrij Lioubartsev
 *  Email: dmitrijl42@gmail.com
 *
 * For more info about this class see ForceFieldJoint.h
 */

#include "ForceFieldJoint.h"

ForceFieldJoint::ForceFieldJoint(string element, vector<string>* bonds, data dat) {
	this->element = element;
	bondTypes = (*bonds);
	numBonds = bondTypes.size();
	this->dat = dat;
}

ForceFieldJoint::~ForceFieldJoint() {
	//do nothing
}

int ForceFieldJoint::getBondCount() {return numBonds;}
string ForceFieldJoint::getElement() {return element;}
ForceFieldJoint::data ForceFieldJoint::getData() {return dat;}

/*
 * Check if this ForceFieldJoint matches a particular Atom's neighbors
 * (atoms its bond to).
 * @param structure Call Atom::getBonds.
 */
bool ForceFieldJoint::matches(vector<Atom*>* structure) {
	if(numBonds == 0) //the base case, number of atoms is "*", or 0 as I define it.
		return true;

	if(numBonds != (int) ((*structure).size())) //Not same amount of bonds
		return false;

	vector<string> cpy = bondTypes; // a cpy of bondTypes.
	/*
	 * Why a copy? Because here is how this works:
	 * Cycle through all bonded elements in the structure to check, and
	 * if found a match in bondTypes (if not by same element, then by '*'),
	 * remove it from the list (thats why its a copy, to not mess with the original).
	 * If "cpy" is empty, it means all elements from "structure" checked out,
	 * and this function should return true. To optimize, instead of letting it go to full,
	 * as soon as a dismatch is found, return false (variable match).
	 */

	for(int i = 0; i < numBonds; i++) {
		const string& e = (*((*structure)[i])).getElement();
		bool star = true;
		bool match = false;
		for(unsigned int j = 0; j < cpy.size(); j++) { //first check by element
			if(cpy[j] == e) {
				cpy.erase(cpy.begin()+j, cpy.begin()+j+1);
				star = false;
				match = true;
				break;
			}
		}
		if(star) //if didnt match by element, try to match by star.
		for(unsigned int j = 0; j < cpy.size(); j++) {
			if(cpy[j] == "*") {
				cpy.erase(cpy.begin()+j, cpy.begin()+j+1);
				match = true;
				break;
			}
		}

		if(!match) //if didnt match, just stop and return.
			break;

	}

	if(cpy.empty())
		return true;
	else
		return false;
}
